#!/bin/sh

#  Author: PAH
# Company: Novell Inc.
# Purpose: To install Novell Security patches on the UNIX platforms.
#
# Code Standard: To allow the script to run on multiplatforms, ensure that
#		 functions follow the following standard:
#
#                # Comment describing what the function does
# 		 functionName()
#		 {
#			doActionsInsideFunction
#		 }

##### Start of general script variables #####
setVars_General()
{
	# Script version and title
	SCRIPT_VERSION_MAJOR=1
	SCRIPT_VERSION_MINOR=0
	SCRIPT_VERSION_REVISION=14
	SCRIPT_VERSION="$SCRIPT_VERSION_MAJOR.$SCRIPT_VERSION_MINOR.$SCRIPT_VERSION_REVISION"
	SCRIPT_TITLE="Patch Installation"

	# Supported Operating Systems for this script
	SUPPORTED_OS="SunOS HPUX Linux AIX"

	# Null Device
	NULLDEV=/dev/null

	# Default CMD Variables
	CMD_HOSTOS="uname -s"

	# Starting Stage number
	STAGE=0

	# Failure Counts
	FAILURE_COUNT=0
	FAILURE_TOTAL=0

	# Temp file name for storing full PKG info.  This is used because it's 
	# quicker reading from disk than querying the native package database 
	# constantly.
	PKG_TEMP_INFO_FILE="/tmp/.novell_temp_pkg-info.$$"
	PKGFILE_TEMP_INFO_FILE="/tmp/.novell_temp_pkgfile_info.$$"
	
	# Save output from a command into this file.  The output can then be used later on in tracking and displaying errors.
	COMMAND_OUTPUT_TEMP_FILE="/tmp/.novell_temp_sp_install_command_output.$$"
		
	# Command to cat the information inside the package temp file
	CMD_PKGINFO="cat $PKG_TEMP_INFO_FILE"

	# Command to cat the information inside the package file temp file
	CMD_PKGFILEINFO="cat $PKGFILE_TEMP_INFO_FILE"
	
	# Command to cat the output from the generic output temp file
	CMD_COMMAND_OUTPUT="cat $COMMAND_OUTPUT_TEMP_FILE"

	# Command to get the current user's ID
	CMD_USER_ID="id"

	# List of processes that will be killed (This will be used to do a grep against ps -ef)
        PROCESS_LIST="ndsrepair ndstrace ndsdump ndsconfig ndsmerge ldapconfig nmasconfig nds_uamcd nds_ssod ConsoleOne ndsd novell/httpd novell/tomcat novell/java"

	# List of temp files that need to be cleaned up
	TEMP_FILES="$PKG_TEMP_INFO_FILE $PKGFILE_TEMP_INFO_FILE $COMMAND_OUTPUT_TEMP_FILE"

	# List of the signals that can be trapped by the script eg: If someone hits CTRL-C then we want to be able to clean up the temp files etc.
	TRAP_SIGNALS="1 2 3 5"

	# Package version not found string
	PKG_VERSION_NOT_FOUND="Not found!"
}
##### End of general script variables #####


##### Start of Product variables #####
setVars_Product()
{
	# Remove Packages
	REMOVE_PACKAGES="true"

	# Regexp for finding all Novell packages
	NOVELL_PACKAGES="NOVL|NDS|NLDAP|novell|ntls|IBMJava2|nici"

	PRODUCT_TITLE="Novell Security Update"
#	PRODUCT_PACKAGES="NOVLnmas|NDSnmas"
	PRODUCT_VERSION_MAJOR=8
	PRODUCT_VERSION_MINOR=0
	PRODUCT_VERSION_REVISION=0
	PRODUCT_VERSION_BUILD=0
#	PRODUCT_VERSION_BASE="$PRODUCT_VERSION_MAJOR.$PRODUCT_VERSION_MINOR"
	PRODUCT_VERSION_BASE="8.0.0.0"
#	PRODUCT_VERSION="$PRODUCT_VERSION_MAJOR.$PRODUCT_VERSION_MINOR.$PRODUCT_VERSION_REVISION.$PRODUCT_VERSION_BUILD"
	PRODUCT_VERSION="$PRODUCT_VERSION_MAJOR"

	# NDSBase package information for pre-req check
	PRODUCT_REQUIRED_NDS_PKG_NAME="NDSserv"
	PRODUCT_REQUIRED_NDS_TITLE="Novell eDirectory"
	PRODUCT_REQUIRED_NDS_PKG_VERSION_MAJOR=8
	PRODUCT_REQUIRED_NDS_PKG_VERSION_MINOR=7
	PRODUCT_REQUIRED_NDS_PKG_VERSION_REVISION=1
	PRODUCT_REQUIRED_NDS_PKG_VERSION_BUILD=0
	PRODUCT_REQUIRED_NDS_PKG_VERSION="$PRODUCT_REQUIRED_NDS_PKG_VERSION_MAJOR.$PRODUCT_REQUIRED_NDS_PKG_VERSION_MINOR.$PRODUCT_REQUIRED_NDS_PKG_VERSION_REVISION"

	# NICI package information for pre-req check
	PRODUCT_REQUIRED_NICI_PKG_NAME="NOVLniu0"
	PRODUCT_REQUIRED_NICI_TITLE="NICI"
	PRODUCT_REQUIRED_NICI_PKG_VERSION_MAJOR=2
	PRODUCT_REQUIRED_NICI_PKG_VERSION_MINOR=6
	PRODUCT_REQUIRED_NICI_PKG_VERSION_REVISION=7
	PRODUCT_REQUIRED_NICI_PKG_VERSION_BUILD=0
	PRODUCT_REQUIRED_NICI_PKG_VERSION="$PRODUCT_REQUIRED_NICI_PKG_VERSION_MAJOR.$PRODUCT_REQUIRED_NICI_PKG_VERSION_MINOR.$PRODUCT_REQUIRED_NICI_PKG_VERSION_REVISION"

	# iManager package information for pre-req check (optional)
	PRODUCT_REQUIRED_IMANAGER_PKG_NAME="NOVLiMgr"
	PRODUCT_REQUIRED_IMANAGER_PKG_EXCLUDE_LIST="NOVLpkis"
	PRODUCT_REQUIRED_IMANAGER_TITLE="Novell iManager"
	PRODUCT_REQUIRED_IMANAGER_PKG_VERSION_MAJOR=2
	PRODUCT_REQUIRED_IMANAGER_PKG_VERSION_MINOR=0
	PRODUCT_REQUIRED_IMANAGER_PKG_VERSION_REVISION=2
	PRODUCT_REQUIRED_IMANAGER_PKG_VERSION_BUILD=0
	PRODUCT_REQUIRED_IMANAGER_PKG_VERSION="$PRODUCT_REQUIRED_IMANAGER_PKG_VERSION_MAJOR.$PRODUCT_REQUIRED_IMANAGER_PKG_VERSION_MINOR.$PRODUCT_REQUIRED_IMANAGER_PKG_VERSION_REVISION"

	PRODUCT_INIT_SCRIPT="/etc/init.d/ndsd"

	PRODUCT_DEFAULT_DIB_DIR="/var/nds/dib"
	PRODUCT_DEFAULT_CONF_DIR="/etc"

	PRODUCT_NDSCONF_FILE="nds.conf"
	PRODUCT_IMONCONF_FILE="ndsimon.conf"
	PRODUCT_NDSDBINI_FILE="_ndsdb.ini"
}
##### End of Product variables #####

##### Start of Host OS specific Variables #####
# Set the Linux specific variables
setVars_Linux()
{
	FULLHOSTNAME="`hostname --long`"
	CMD_SINGLE_PKG_FULL_INFO="rpm -q --info"
	CMD_SINGLE_PKG_INSTALL_CHECK="rpm -q"
	CMD_LIST_ALL_PACKAGES="rpm -qa"
	CMD_PKG_NAME_EXTRACT="grep -v \"\""
	CMD_SINGLE_PKG_REMOVE="rpm --erase --nodeps --noscripts"
	CMD_SINGLE_PKG_INSTALL="rpm --install --force --nodeps"
	CMD_SINGLE_PKGFILE_FULL_INFO="rpm -q --info -p"

	# This var just allows storeFullPKGFileInfoIntoTempFile() to not append the package name
	# to the command $CMD_SINGLE_PKGFILE_FULL_INFO when extracting package info directly from
	# a package file.

	# This var just allows installPackages() to not pass the $PKGFILE_PKG_NAME var to the package install utililty.
	CMD_SINGLE_PKG_INSTALL_INGORE_PKG_NAME="true"
	PRODUCT_REQUIRED_NICI_PKG_NAME="nici"
	PRODUCT_REQUIRED_IMANAGER_PKG_NAME="novell-imanager"

	# Because of an issue with eDirectory 8.7.1 on SuSE this
	# startup script will work for RedHat and SuSE. 8.7.3 
	# will still use /etc/init.d/ndsd
	PRODUCT_INIT_SCRIPT_871="/etc/rc.d/init.d/ndsd"
}

# Set the SunOS specific variables
setVars_SunOS()
{
	FULLHOSTNAME="`hostname`.`domainname`"
	CMD_SINGLE_PKG_FULL_INFO="pkgparam -v"
        CMD_SINGLE_PKG_INSTALL_CHECK="pkginfo"
	CMD_LIST_ALL_PACKAGES="pkginfo"
	CMD_PKG_NAME_EXTRACT="cut -b 13-27"
	CMD_SINGLE_PKGFILE_FULL_INFO="pkgparam -vd"

	SUNOS_PKG_ADMIN_FILE="/tmp/.novell-temp-pkg-sunos-admin.$$"
	CMD_SINGLE_PKG_INSTALL="pkgadd -a $SUNOS_PKG_ADMIN_FILE -d"
        CMD_SINGLE_PKG_REMOVE="pkgrm -n -a $SUNOS_PKG_ADMIN_FILE"

        # Create the admin file for installing and removing packages without intervention
        echo '#ident \"@(#)default  1.4 92/12/23 SMI\" /* SVr4.0  1.5.2.1 */' > $SUNOS_PKG_ADMIN_FILE
        echo "mail="                    >> $SUNOS_PKG_ADMIN_FILE
        echo "instance=unique"          >> $SUNOS_PKG_ADMIN_FILE
        echo "partial=ask"              >> $SUNOS_PKG_ADMIN_FILE
        echo "runlevel=ask"             >> $SUNOS_PKG_ADMIN_FILE
        echo "idepend=nocheck"          >> $SUNOS_PKG_ADMIN_FILE
        echo "rdepend=nocheck"          >> $SUNOS_PKG_ADMIN_FILE
        echo "space=ask"                >> $SUNOS_PKG_ADMIN_FILE
        echo "setuid=ask"               >> $SUNOS_PKG_ADMIN_FILE
        echo "conflict=nocheck"         >> $SUNOS_PKG_ADMIN_FILE
        echo "action=nocheck"           >> $SUNOS_PKG_ADMIN_FILE
        echo "basedir=default"          >> $SUNOS_PKG_ADMIN_FILE
}

# Set the HP-UX specific variables
setVars_HPUX()
{
	FULLHOSTNAME="`hostname`.`domainname`"
	CMD_SINGLE_PKG_FULL_INFO="swlist -v"
        CMD_SINGLE_PKG_INSTALL_CHECK="swlist"
	CMD_LIST_ALL_PACKAGES="swlist -a product"
	CMD_PKG_NAME_EXTRACT="grep -v \"\""
	CMD_SINGLE_PKG_REMOVE="swremove -x enforce_dependencies=false"

	CMD_SINGLE_PKG_INSTALL="swinstall -s"
	CMD_SINGLE_PKGFILE_FULL_INFO="swlist -v -d -s "
	PRODUCT_INIT_SCRIPT="/sbin/init.d/ndsd"
}

# Set the AIX specific variables
setVars_AIX()
{
	FULLHOSTNAME="`hostname`.`domainname`"
	CMD_SINGLE_PKG_FULL_INFO="lslpp -la"
        CMD_SINGLE_PKG_INSTALL_CHECK="lslpp -l"
	CMD_LIST_ALL_PACKAGES="lslpp -l -c"
	CMD_PKG_NAME_EXTRACT="cut -d : -f 2"
	CMD_SINGLE_PKG_REMOVE="installp -ug"
	CMD_SINGLE_PKGFILE_FULL_INFO="installp -ld"
#	CMD_SINGLE_PKG_INSTALL="installp -acgXd"
	CMD_SINGLE_PKG_INSTALL="installp -acFXd"
	REMOVE_PACKAGES="false"

	# This var just allows storeFullPKGFileInfoIntoTempFile() to not append the package name
	# to the command $CMD_SINGLE_PKGFILE_FULL_INFO when extracting package info directly from
	# a package file.
	CMD_SINGLE_PKGFILE_FULL_INFO_IGNORE_PKG_NAME="true"

	PRODUCT_INIT_SCRIPT="/etc/ndsd"
	PRODUCT_REQUIRED_NDS_PKG_NAME="NDS.NDSbase"
}
##### End of Host OS Variables #####

##### Set the ANSI colours #####
# Enable or disable ANSI colours
# Input: $1="ON" or "OFF"
setANSIColours()
{
	if [ -z "$1" ]
	then
		dispError "Internal Error:  setANSIColours() was not passed a parameter."
		abortScript "1"
	else
		case $1 in
		"on" |"ON")
			enableANSIColours;;
		"off" |"OFF")
			disableANSIColours;;
		*)
			dispError "Internal Error:  setANSIColours() was passed an invalid parameter of '$1'."
			abortScript "1"
		esac
	fi
}

enableANSIColours()
{
  esc=""

  fgblack="${esc}[30m";   fgred="${esc}[31m";    fggreen="${esc}[32m";
  fgyellow="${esc}[33m";  fgblue="${esc}[34m";   fgpurple="${esc}[35m";
  fgcyan="${esc}[36m";    fgwhite="${esc}[37m";
  
  bgblack="${esc}[40m";   bgred="${esc}[41m";   bggreen="${esc}[42m";
  bgyellow="${esc}[43m";  bglue="${esc}[44m";   bgpurple="${esc}[45m";
  bgcyan="${esc}[46m";    bgwhite="${esc}[47m";

  boldon="${esc}[1m";    boldoff="${esc}[22m";
  italicson="${esc}[3m"; italicsoff="${esc}[23m";
  ulon="${esc}[4m";      uloff="${esc}[24m";
  invon="${esc}[7m";     invoff="${esc}[27m";

  reset="${esc}[0m";
}

# Disable the ANSI colours
disableANSIColours()
{

  fgblack="";   fgred="";    fggreen="";
  fgyellow="";   fgblue="";   fgpurple="";
  fgcyan="";    fgwhite="";
  
  bgblack="";   bgred="";   bggreen="";
  bgyellow="";   bglue="";   bgpurple="";
  bgcyan="";    bgwhite="";

  boldon="";    boldoff="";
  italicson=""; italicsoff="";
  ulon="";      uloff="";
  invon="";     invoff="";

  reset="";
}

# Reset the ANSI colours on the terminal back to normal
resetANSIcolours()
{
	echo ${reset}
}
##### End of the ANSI colours #####


##### Start of Auxillary Functions #####
# Determine which OS the script is being run on
# Output: OS contains operating system
getHostOS()
{
	OS="`$CMD_HOSTOS`"

	# Convert HP-UX to HPUX as function or variable names don't like hyphens in them
	if [ "$OS" = "HP-UX" ]
	then
		OS="HPUX"
	fi
}

# Verify the the host OS that this script is executing on is supported
# by the script
verifySupportedOS()
{
	for CHECKOS in $SUPPORTED_OS
        do
		if [ "$OS" = "$CHECKOS" ]
              	then
			return 1
		fi
	done
	echo "Sorry, the \"$OS\" operating system is not supported by this installation script."
	exit 1
}


# Delete the temp files that have been created.  Ensure that any temp file that needs to be deleted is in $TEMP_FILES from setVars_General().
removeTempFiles()
{
	for TEMP_FILE_NAME in $TEMP_FILES
	do
		rm -rf $TEMP_FILE_NAME		
	done
}

# Calculates the length of a string
# Input: $1=string to calculate
# Output: LENGTH= number of chars in string from $1
length()
{
	LENGTH="`echo $1| wc -c`"
	# remove one from the total due to the CR at the end
	LENGTH="`expr $LENGTH - 1`"
}

# Register for signal trapping.  This is used so that the temp files can be cleaned up.
registerSignalTraps()
{
	trap 'abortScriptFromUser' $TRAP_SIGNALS
}

# Aborts the script because the user requested to. This function is what the signal traps are registered for.
abortScriptFromUser()
{
	echo
	abortScript "99" "YES"
}

# Abort the script.  
# Input:  $1=OPTIONAL! exit code 
#         $2=OPTIONAL! (requires $1) "YES" = Really abort - ignore whatever the FORCE switch is set to 
abortScript()
{
	SCRIPT_ABORTED_MSG="${fgred}Script execution has been aborted.${reset}"
	SCRIPT_NOT_ABORTED_MSG="${boldon}${fgred}Script was not aborted due to FORCE installation enabled on the command line.${reset}"

	if [ -z "$1" ]
	then
		EXIT_CODE=0
	else
		EXIT_CODE=$1
	fi

	if [ "$FORCE" = "true" ]
	then
		if [ "$2" != "YES" ]
		then
			echo $SCRIPT_NOT_ABORTED_MSG
		else
			removeTempFiles
			echo $SCRIPT_ABORTED_MSG
			exit $EXIT_CODE
		fi
	else
		removeTempFiles
		echo $SCRIPT_ABORTED_MSG
		exit $EXIT_CODE
	fi
}

# Display the script title
dispTitle()
{
	clear
	echo "${reset}${boldon}+------------------------------------------------------------------------------+"
	printf "| %55s %22s\n" "$PRODUCT_TITLE $SCRIPT_TITLE" "|"	
	printf "| Product Version: %-10s     OS: %-10s     Script Version: %-10s|\n" "$PRODUCT_VERSION" "$OS" "$SCRIPT_VERSION"
	echo "+------------------------------------------------------------------------------+${reset}"
}

# Display the message informing the installater to backup their files first
dispInstallationPreparationMessage()
{
	echo "+------------------------------------------------------------------------------+"
	echo "|                         Installation Preparation                             |"
	echo "+------------------------------------------------------------------------------+"
	echo "|                                                                              |"
	printf "|  This script will install %-50s |\n" "\"$PRODUCT_TITLE $PRODUCT_VERSION\"."
	echo "|                                                                              |"
#	echo "|  You should have a current working backup of these files and directories:    |"
#	echo "|                                                                              |"
#	printf "|  1.  %-50s                      |\n" "$PRODUCT_CONF_DIR/$PRODUCT_NDSCONF_FILE"
#	printf "|  2.  %-50s                      |\n" "$PRODUCT_DIB_DIR/*"
#	printf "|  3.  %-50s                      |\n" "$PRODUCT_CONF_DIR/$PRODUCT_IMONCONF_FILE"
#	echo "|                                                                              |"
	echo "|  Please ensure that you have backups of eDirectory before continuing.        |"
	echo "|                                                                              |"
	printf "|  ${fgred}WARNING:  %s will be shutdown during the patch installation.${reset} |\n" "Novell eDirectory"
	echo "|    ${fgred}        Ensure that all eDirectory activities are stopped before${reset}          |"
	echo "|    ${fgred}        continuing with the installation of this patch.${reset}                   |"
	echo "|    ${fgred}        If iManager is running on this server then it will also           ${reset}|"
	echo "|    ${fgred}        be shut down.                                                     ${reset}|"
	echo "+------------------------------------------------------------------------------+"
	echo
	echo
	echo
	echo

}

# Display a message stating that the install completed and how to start the product back up again.
dispInstallCompletedSuccessfullyMessage()
{
	dispNote "The $SCRIPT_TITLE of $PRODUCT_TITLE $PRODUCT_VERSION completed sucessfully.\nTo start eDirectory use \"${fgyellow}$PRODUCT_INIT_SCRIPT start${fgcyan}\"."
}

# Prompt the user for Yes or No
# Input: $1= Prompt message
promptForYesOrNo()
{
	printf "%s" "${boldon}${fgyellow}$1 ${fgwhite}[y/N]${reset} "
	INPUT=
	read INPUT

	case $INPUT in
		"y" |"Y")
			return 0;;
		*)
			echo
			echo "You did not agree to proceed with the installation."
			abortScript "0" "YES"
	esac
}

# Display a message letting the user know they have agree'd to proceed
dispProceedMessage()
{
	echo
	echo "You have agreed to proceed with the installation."
	echo "Applying the patches to \"$FULLHOSTNAME\"."
	echo
}

# Display a message informing the installer that they have enabled the FORCE 
# option via the command line (if they have done so).
dispForceMessageIfRequired()
{
	if [ "$FORCE" = "true" ]
	then
		echo "    ${boldon}${bgred}${fgwhite}*** WARNING: The \"force\" option has been enabled at the command line ***${reset}"
	else
		echo
	fi
}

# Display an error message. It will 'bordered' by equal signs (eg: ====). 
# If "NO_ERRORS" is true then don't display the error. Not really a good idea
# as you won't know why the error occured.
dispError()
{
	if [ "$NO_ERRORS" != "true" ]
	then
		printf "%s${fgred}==================================== ERROR =====================================${reset}" ""
		printf "%s${fgred}\n$1${reset}\n" ""
		printf "%s${fgred}================================================================================${reset}" ""
		echo
	fi
}

# Display a note on the screen to the user. It will 'bordered' by equal signs (eg: ====). 
dispNote()
{
	printf "%s${fgcyan}${boldon}==================================== NOTE ======================================${reset}" ""
	printf "%s${fgcyan}${boldon}\n$1${reset}\n" ""
	printf "%s${fgcyan}${boldon}================================================================================${reset}" ""
	echo
}

# Get the output from a command from the $COMMAND_OUTPUT_TEMP_FILE 
# Output: ERROR_OUTPUT= text inside $COMMAND_OUTPUT_TEMP_FILE
getCommandOutput()
{
	ERROR_OUTPUT="`$CMD_COMMAND_OUTPUT`"
}

# Get the vars for the product. Eg: The DIB DIR if it has been changed from the default
getVars_Product()
{
	# Set the locations for the product config files
        PRODUCT_CONF_DIR=$PRODUCT_DEFAULT_CONF_DIR
        PRODUCT_DIB_DIR=$PRODUCT_DEFAULT_DIB_DIR
 
	# If the default config file exists then change the above vars
	if [ -f "$PRODUCT_DEFAULT_CONF_DIR/$PRODUCT_NDSCONF_FILE" ]
        then
                PRODUCT_CONF_DIR="`cat $PRODUCT_DEFAULT_CONF_DIR/$PRODUCT_NDSCONF_FILE | grep -v \"^#\" | grep n4u.server.configdir|cut -d = -f 2`"
                if [ -z "$PRODUCT_CONF_DIR" ]
                then
                        PRODUCT_CONF_DIR=$PRODUCT_DEFAULT_CONF_DIR
                fi

                PRODUCT_DIB_DIR="`cat $PRODUCT_DEFAULT_CONF_DIR/$PRODUCT_NDSCONF_FILE | grep -v \"^#\" | grep n4u.nds.dibdir|cut -d = -f 2`"
                if [ -z "$PRODUCT_DIB_DIR" ]
                then
                        PRODUCT_DIB_DIR=$PRODUCT_DEFAULT_DIB_DIR
                fi
	fi
}
##### End of Auxillary Functions #####

##### Start of Stage Functions #####
# Display the result on the right hand side of the screen - but on the left
# Input: $1=Info to display - MAX Chars=13
dispResult()
{
	printf "${boldon}[${fggreen}%-13s${reset}${boldon}]${reset}\n" "$1"
}

# Display the result of the right hand side of the screen - but on the right
# Input: $1=Info to display - MAX Chars=13. The colour is Green for good.
dispResultNumber()
{
	printf "${boldon}[${fggreen}%13s${reset}${boldon}]${reset}\n" "$1"
}

# Display the result of the right hand side of the screen - but on the right
# Input: $1=Info to display - MAX Chars=13.  The colour is Red for Failed.
dispResultNumberFailed()
{
	printf "${boldon}[${fgred}%13s${reset}${boldon}]${reset}\n" "$1"
}

# Display the item of the left hand side of the screen
# Input: $1=Item to display
#        $2=OPTIONAL! Extra info to display if it is given
#	 $3=OPTIONAL! "YES" Extra param to move 2nd column right 3 spaces to
#                     give room room to long items. ie: NMAS Linux file name
dispItem()
{
	if [ -z "$2" ]
	then
		printf "${boldon}| %-63s${reset}" "$1"
	else
		if [ -z "$3" ]
		then
			printf "${boldon}| %-38s%-25s${reset}" "$1" "| $2" 
		else
			if [ "$3" = "YES" ]
			then
				printf "${boldon}| %-42s%-21s${reset}" "$1" "| $2"
			fi
		fi
	fi

}

# Display the stage status and add the previous stages info to the summary results
# Input: $1=Stage Description
dispStage()
{
	STAGE="`expr $STAGE + 1`"

	STAGE_DESCRIPTION="`printf \"Stage [%02d]: ${fgyellow}%-64s${fgwhite}\n\" \"$STAGE\" \"$1\"`"

	echo "${boldon}"
	echo "+------------------------------------------------------------------------------+"
	echo "| $STAGE_DESCRIPTION |" 
	echo "+------------------------------------------------------------------------------+${reset}"
}

# Increment the number of failures
failure()
{
	FAILURE_COUNT="`expr $FAILURE_COUNT + 1`"
}

# Stores the failure counts for each stage
recordStageFailureTotals()
{
	FAILURE_STAGES_COUNT="$FAILURE_STAGES_COUNT $FAILURE_COUNT"
}

# Display the header for the section being processed
# Input: $1=First column header
#        $2=OPTIONAL! Second column header
#        $3=OPTIONAL! "YES" Extra param to move 2nd column right 3 spaces to
#                     give room room to long items. ie: NMAS Linux file name
dispHeader()
{
	if [ -z "$2" ]
	then
		printf "${boldon}| %-63s%-13s\n" "$1" "| Result      |"	
	else
		if [ -z "$3" ]
		then
			printf "${boldon}| %-38s%-25s%15s\n" "$1" "| $2" "| Result      |"	
		else
			if [ "$3" = "YES" ]
			then
				printf "${boldon}| %-42s%-21s%15s\n" "$1" "| $2" "| Result      |"					
			fi
		fi
	fi

	echo "+------------------------------------------------------------------------------+${reset}"
}

# Start of a stage
# Input: $1=Name of stage
stageStart()
{
	dispStage "$1"
}

# End of a stage.  If any failures occured that were set by failure() then the stage is determined to have FAILED
# and will show the Summary info which then exits the script.
stageEnd()
{
	recordStageFailureTotals

	echo "${boldon}+------------------------------------------------------------------------------+${reset}"
	ITEM_DESC="`printf \"*** Stage [%02d] Result\" $STAGE`"
	dispItem "$ITEM_DESC"

	if [ "$FAILURE_COUNT" != "0" ]
	then
		dispResult "${reset}${fgred}${boldon}   FAILED    ${reset}"
		echo "${boldon}+------------------------------------------------------------------------------+${reset}"
		if [ "$FORCE" != "true" ]
		then
			dispSummaryInfo
		else
			dispNote "Stage $STAGE did not complete successfully. However, patch installation will continue\ndue to the \"force\" switch being used!"
		fi
	else
		dispResult "${reset}${fggreen}${boldon}  COMPLETE   ${reset}"
		echo "${boldon}+------------------------------------------------------------------------------+${reset}"
	fi

	FAILURE_COUNT=0
}

# Abort out of a stage, display the summary info and exit the script (the exit is actually inside dispSummaryInfo())
stageAbort()
{
        recordStageFailureTotals
        FAILURE_COUNT=0
	dispSummaryInfo
}

# Display the summary info
dispSummaryInfo()
{
	echo "${boldon}"
	echo "+------------------------------------------------------------------------------+"
	echo "+                                Results Summary                               +"
	echo "+------------------------------------------------------------------------------+${reset}"

	STAGE_NUMBER=0
	for STAGE_FAILURE_COUNT in $FAILURE_STAGES_COUNT
	do
		STAGE_NUMBER="`expr $STAGE_NUMBER + 1`"
		STAGE_NAME="Stage `printf \"[%02d]\" \"$STAGE_NUMBER\"`"
		FAILURE_TOTAL="`expr $FAILURE_TOTAL + $STAGE_FAILURE_COUNT`"

		dispItem "$STAGE_NAME Errors"
                if [ "$STAGE_FAILURE_COUNT" = "0" ]
                then
                        dispResult "    NONE     "
                else
                        dispResultNumberFailed "$STAGE_FAILURE_COUNT"
                fi
	done

	dispItem "Total Errors"
	if [ "$FAILURE_TOTAL" = "0" ]
        then
                dispResult "    NONE     "
        else
                dispResultNumberFailed "$FAILURE_TOTAL"
        fi

	echo "${boldon}|                                                                              |${reset}"

	# If FORCE was enabled then the final end result (most likely a FAILED) may not really be completly accurate
	if [ "$FORCE" = "true" ]
	then
		dispNote "NOTE: The \"force\" switch has been enabled. The Final End Result status may not \nbe completly accurate. Run ${fgyellow}$0 --showinstalled${fgcyan} to display installed\npackages from this patch."
	fi

	echo "${boldon}+------------------------------------------------------------------------------+${reset}"
	dispItem "Final $PRODUCT_TITLE $PRODUCT_VERSION $SCRIPT_TITLE Status"
	if [ "$FAILURE_TOTAL" = "0" ]
	then
		dispResult "${reset}${bggreen}${boldon}${fgwhite}  INSTALLED  ${reset}"
		RC=0
	else
		dispResult "${reset}${bgred}${fgblack}   FAILED    ${reset}"
		RC=1
	fi

	echo "${boldon}+------------------------------------------------------------------------------+${reset}"

	# If the install displayed INSTALLED for the final message then show the message regarding how to start up the product.
	if [ "$RC" = "0" ]
	then
		dispInstallCompletedSuccessfullyMessage
	fi

	# Delete the temp files that have been created
	removeTempFiles
	resetANSIcolours
	exit $RC
}
##### End of Stage Functions #####

##### Start of Various Action Functions #####
# Display the status of the packages: eg: INSTALLED, NOT INSTALLED
showPackagesInstallStatus()
{
	# Display header
	dispHeader "Package Name" "Package Version"

	for PKG in $PKG_LIST
	do

		getPackageVersionInfo "$PKG" 
		# The extra space " " for the 2nd param for the dispItem() function is needed so that if a 
		# package version was not found (because the package isn't installed etc) then still pass at
		# least a space char to dispItem() so that it will display the 2nd column bar seperator | 
		# and the column will just look to be "blank"
		dispItem "$PKG" "$PKG_VERSION "

		INSTALLED=
		# Command to check if a single package is installed, redirecting
		# stdout and errout to the null device
		$CMD_SINGLE_PKG_INSTALL_CHECK $PKG 1>$NULLDEV 2>$NULLDEV
		INSTALLED=$?
		if [ "$INSTALLED" = "0" ]
		then
			PKGSTATUS="  INSTALLED  "
		else
			PKGSTATUS="${fgred}NOT INSTALLED${reset}"
		fi

		# Display the package status result
		dispResult "$PKGSTATUS"

	done
}

# Display all information for package
dispPackageFullInfo()
{
	echo "`$CMD_SINGLE_PKG_FULL_INFO \"$1\"`"
}

# Clear the PKG_VERSION variables
clearPKGVersion()
{
	PKG_VERSION_MAJOR=
	PKG_VERSION_MINOR=
	PKG_VERSION_REVISION=
	PKG_VERSION_BUILD=
	PKG_VERSION_RELEASE=
	PKG_VERSION=
}

# Clear the PKGFILE_VERSION variables
clearPKGFileVersion()
{
	PKGFILE_VERSION_MAJOR=
	PKGFILE_VERSION_MINOR=
	PKGFILE_VERSION_REVISION=
	PKGFILE_VERSION_BUILD=
	PKGFILE_VERSION_RELEASE=
	PKGFILE_VERSION_BASE=
	PKGFILE_VERSION=
}

# Get the package version information
# Input: $1=Package name
# Output: PKG_VERSION_MAJOR, PKG_VERSION_MINOR
#         PKG_VERSION_REVISION, PKG_VERSION_BUILD
#         PKG_VERSION   (In format of major.minor.revision.build)
# The PKG_VERSION variables are set in getPackageVersionInfo_$OS
getPackageVersionInfo()
{
	clearPKGVersion
	if [ -z "$1" ]
	then
		dispError "Internal Error: No package name provided to function 'getPackageVersionInfo()'"
		abortScript "1" 
	else
	        storeFullPKGInfoIntoTempFile "$1"
		getPackageVersionInfo_$OS "$1"
		getFullPKGVersion
	fi
}

# Get the package file version information
# Input: $1=Package file name
# Output: PKGFILE_VERSION_MAJOR, PKGFILE_VERSION_MINOR
#         PKGFILE_VERSION_REVISION, PKGFILE_VERSION_BUILD, PKGFILE_VERSION_RELEASE
#         PKGFILE_VERSION_BASE, PKGFILE_VERSION   (In format of major.minor.revision.build)

# The PKGFILE_VERSION_* variables are actually set in getPackageFileVersionInfo_$OS
getPackageFileVersionInfo()
{
	clearPKGFileVersion
	if [ -z "$1" ]
	then
		dispError "Internal Error: No package filename provided to function 'getPackageFileVersionInfo()'"
		abortScript "1"
	else
		getPackageNameFromPackageFile "$1"
	        storeFullPKGFileInfoIntoTempFile "$1" "$PKGFILE_PKG_NAME"
		getPackageFileVersionInfo_$OS "$1" 
		getFullPKGFileVersion
	fi
}


# Get the package name given the package file name
# Input: $1=package file name
# Output: $PKGFILE_PKG_NAME = name of package
getPackageNameFromPackageFile()
{
	if [ -z "$1" ]
        then
                dispError "Internal Error: No package filename provided to function getPackageNameFromPackageFile()"
		abortScript "1"
        else
		getPackageNameFromPackageFile_$OS "$1" "$2"
	fi
}

# SunOS:  Get the Major, Minor, Revision and Build information for the package
getPackageFileVersionInfo_SunOS()
{
	PKGFILE_VERSION_MAJOR="`$CMD_PKGFILEINFO | grep VERSION | tr -d \' | tr - . | cut -d= -f 2 | cut -d . -f 1`"
	PKGFILE_VERSION_MINOR="`$CMD_PKGFILEINFO | grep VERSION | tr -d \' | tr - . | cut -d= -f 2 | cut -d . -f 2`"
	PKGFILE_VERSION_REVISION="`$CMD_PKGFILEINFO | grep VERSION | tr -d \' | tr - . | cut -d= -f 2 | cut -d . -f 3`"
	PKGFILE_VERSION_BUILD="`$CMD_PKGFILEINFO | grep VERSION | tr -d \' | tr - . | cut -d= -f 2 | cut -d . -f 4`"
}

# HPUX:  Get the Major, Minor, Revision and Build information for the package
getPackageFileVersionInfo_HPUX()
{
	PKGFILE_VERSION_MAJOR="`$CMD_PKGFILEINFO | egrep \"^revision\" | head -1 | awk -F\" \" '{print $2}'|cut -d . -f 1`"
	PKGFILE_VERSION_MINOR="`$CMD_PKGFILEINFO | egrep \"^revision\" | head -1 | awk -F\" \" '{print $2}'|cut -d . -f 2`"
	PKGFILE_VERSION_REVISION="`$CMD_PKGFILEINFO | egrep \"^revision\" | head -1 | awk -F\" \" '{print $2}'|cut -d . -f 3`"
	PKGFILE_VERSION_BUILD="`$CMD_PKGFILEINFO | egrep \"^revision\" | head -1 | awk -F\" \" '{print $2}'|cut -d . -f 4`"
}

# AIX:  Get the Major, Minor, Revision and Build information for the package
getPackageFileVersionInfo_AIX()
{
	PKGFILE_VERSION_MAJOR="`$CMD_PKGFILEINFO | head -n 3 | sort -r | head -1 | awk -F\" \" '{ print $2}' | cut -d . -f 1`"
	PKGFILE_VERSION_MINOR="`$CMD_PKGFILEINFO | head -n 3 | sort -r | head -1 | awk -F\" \" '{ print $2}' | cut -d . -f 2`"
	PKGFILE_VERSION_REVISION="`$CMD_PKGFILEINFO | head -n 3 | sort -r | head -1 | awk -F\" \" '{ print $2}' | cut -d . -f 3`"
	PKGFILE_VERSION_BUILD="`$CMD_PKGFILEINFO | head -n 3 | sort -r | head -1 | awk -F\" \" '{ print $2}' | cut -d . -f 4`"
}

# Linux:  Get the Major, Minor, Revision and Build information for the package
getPackageFileVersionInfo_Linux()
{

	PKGFILE_VERSION_MAJOR="`$CMD_PKGFILEINFO | grep Version | head -1 | awk -F" " '{print $3}'|cut -d . -f 1`"
	PKGFILE_VERSION_MINOR="`$CMD_PKGFILEINFO | grep Version | head -1 | awk -F" " '{print $3}'|cut -d . -f 2`"
	PKGFILE_VERSION_REVISION="`$CMD_PKGFILEINFO | grep Version | head -1 | awk -F" " '{print $3}'|cut -d . -f 3`"
	PKGFILE_VERSION_BUILD="`$CMD_PKGFILEINFO | grep Version | head -1| awk -F" " '{print $3}' | cut -d . -f 4 `"

	if [ ! -z "$PKGFILE_VERSION_MAJOR" ]
	then
		if [ -z "$PKGFILE_VERSION_BUILD" ]
		then
			PKGFILE_VERSION_BUILD="0"
		fi
	fi

	PKGFILE_VERSION_RELEASE="`$CMD_PKGFILEINFO | grep Release | head -1| awk -F" " '{print $3}'`"

}

# Extract the package name from the package file
# Input: $1=package file name
# Output: PKGFILE_PKG_NAME=package name
getPackageNameFromPackageFile_SunOS()
{
	PKGFILE_PKG_NAME="`echo $1 | cut -d / -f3 | cut -d . -f 1`"
}

# Extract the package name from the package file
# Input: $1=package file name
# Output: PKGFILE_PKG_NAME=package name
getPackageNameFromPackageFile_HPUX()
{
	PKGFILE_PKG_NAME="`echo $1 | cut -d / -f3 | cut -d . -f 1`"
}

# Extract the package name from the package file
# Input: $1=package file name
# Output: PKGFILE_PKG_NAME=package name
getPackageNameFromPackageFile_AIX()
{
	PKGFILE_PKG_NAME="NDS.`echo $1 | cut -d / -f3 | cut -d . -f 2`"

	# This is a fix for NMAS as the packages were renamed to 
	# NOVLnmas.fileset
	if [ "$PKGFILE_PKG_NAME" = "NDS.fileset" ]
	then
 		PKGFILE_PKG_NAME="`echo $1 | cut -d / -f3 | cut -d . -f 1`"
	fi
}

# Extract the package name from the package file
# Input: $1=package file name
# Output: PKGFILE_PKG_NAME=package name
getPackageNameFromPackageFile_Linux()
{
	PKGFILE_PKG_NAME="`echo $1 | cut -d / -f3 | cut -d - -f 1`"
}

# AIX:  Get the Major, Minor, Revision and Build information for the package
getPackageVersionInfo_AIX()
{
	PKG_VERSION_MAJOR="`$CMD_PKGINFO | grep $1 | head -1 | awk -F\" \" '{print $2}'|cut -d : -f 1| cut -d . -f 1`"
	PKG_VERSION_MINOR="`$CMD_PKGINFO | grep $1 | head -1 | awk -F\" \" '{print $2}'|cut -d : -f 1| cut -d . -f 2`"
	PKG_VERSION_REVISION="`$CMD_PKGINFO | grep $1 | head -1 | awk -F\" \" '{print $2}'|cut -d : -f 1| cut -d . -f 3`"
	PKG_VERSION_BUILD="`$CMD_PKGINFO | grep $1 | head -1 | awk -F\" \" '{print $2}'|cut -d : -f 1| cut -d . -f 4`"
}

# SunOS:  Get the Major, Minor, Revision and Build information for the package
getPackageVersionInfo_SunOS()
{
	PKG_VERSION_MAJOR="`$CMD_PKGINFO | grep VERSION | tr -d \' | tr - . | cut -d= -f 2 | cut -d . -f 1`"
	PKG_VERSION_MINOR="`$CMD_PKGINFO | grep VERSION | tr -d \' | tr - . | cut -d= -f 2 | cut -d . -f 2`"
	PKG_VERSION_REVISION="`$CMD_PKGINFO | grep VERSION | tr -d \' | tr - . | cut -d= -f 2 | cut -d . -f 3`"
	PKG_VERSION_BUILD="`$CMD_PKGINFO | grep VERSION | tr -d \' | tr - . | cut -d= -f 2 | cut -d . -f 4`"
}


# Linux:  Get the Major, Minor, Revision and Build information for the package
getPackageVersionInfo_Linux()
{
	PKG_VERSION_MAJOR="`$CMD_PKGINFO | grep Version | head -1 | awk -F" " '{print $3}'|cut -d . -f 1`"
	PKG_VERSION_MINOR="`$CMD_PKGINFO | grep Version | head -1 | awk -F" " '{print $3}'|cut -d . -f 2`"
	PKG_VERSION_REVISION="`$CMD_PKGINFO | grep Version | head -1 | awk -F" " '{print $3}'|cut -d . -f 3`"
	PKG_VERSION_BUILD="`$CMD_PKGINFO | grep Version | head -1| awk -F" " '{print $3}' | cut -d . -f 4 `"

	if [ ! -z "$PKG_VERSION_MAJOR" ]
	then
		if [ -z "$PKG_VERSION_BUILD" ]
		then
			PKG_VERSION_BUILD="0"
		fi
	fi

	PKG_VERSION_RELEASE="`$CMD_PKGINFO | grep Release | head -1| awk -F" " '{print $3}'`"
}

# HPUX:  Get the Major, Minor, Revision and Build information for the package
getPackageVersionInfo_HPUX()
{
	PKG_VERSION_MAJOR="`$CMD_PKGINFO | egrep \"^revision\" | head -1 | awk -F\" \" '{print $2}'|cut -d . -f 1`"
	PKG_VERSION_MINOR="`$CMD_PKGINFO | egrep \"^revision\" | head -1 | awk -F\" \" '{print $2}'|cut -d . -f 2`"
	PKG_VERSION_REVISION="`$CMD_PKGINFO | egrep \"^revision\" | head -1 | awk -F\" \" '{print $2}'|cut -d . -f 3`"
	PKG_VERSION_BUILD="`$CMD_PKGINFO | egrep \"^revision\" | head -1 | awk -F\" \" '{print $2}'|cut -d . -f 4`"
}


# Dumps the full package info into a temp file.  This is then used for query
# the database rather than making constant calls to the package management
# system native to the OS.
# Input: $1=package name
# Output: Package info is stored in file from $PKG_TEMP_INFO_FILE
storeFullPKGInfoIntoTempFile()
{
	rm -rf $PKG_TEMP_INFO_FILE
	$CMD_SINGLE_PKG_FULL_INFO $1 1>$PKG_TEMP_INFO_FILE 2>$NULLDEV
}



# Dumps the full package file info into a temp file.  This is then used for query
# the database rather than making constant calls to the package management
# system native to the OS.
# Input: $1=package file name
#        $2=OPTIONAL! This is the package name stored in the package file
# Output: Package info is stored in file from $PKGFILE_TEMP_INFO_FILE
storeFullPKGFileInfoIntoTempFile()
{
	rm -rf $PKGFILE_TEMP_INFO_FILE
	PACKAGE_FILE_NAME=$2
	
	# If $CMD_SINGLE_PKGFILE_FULL_INFO_IGNORE_PKG_NAME is true then set $2 back to nothing, as the command
	# for this OS doesn't require the package name to be passed to the line to dump the package info out from
	# the actual file. 
	if [ "$CMD_SINGLE_PKGFILE_FULL_INFO_IGNORE_PKG_NAME" = "true" ]
	then	
		PACKAGE_FILE_NAME=
	fi	

	$CMD_SINGLE_PKGFILE_FULL_INFO `pwd`/$1 $PACKAGE_FILE_NAME 1>$PKGFILE_TEMP_INFO_FILE 2>$PKGFILE_TEMP_INFO_FILE
}


# Concatanate PKG_VERSION_MAJOR, PKG_VERSION_MINOR, PKG_VERSION_REVISION and
# PKG_VERSION_BUILD together to form PKG_VERSION
getFullPKGVersion()
{
	PKG_VERSION=
	PKG_VERSION_BASE=

	if [ -z "$PKG_VERSION_MAJOR" ]
	then
 		PKG_VERSION_MAJOR="0"
	fi

	if [ -z "$PKG_VERSION_MINOR" ]
	then
 		PKG_VERSION_MINOR="0"
	fi

	if [ -z "$PKG_VERSION_REVISION" ]
	then
 		PKG_VERSION_REVISION="0"
	fi

	if [ -z "$PKG_VERSION_BUILD" ]
	then
 		PKG_VERSION_BUILD="0"
	fi

	if [ "$PKG_VERSION_MAJOR" != "" ]
        then
                PKG_VERSION="$PKG_VERSION_MAJOR"
	fi

	if [ "$PKG_VERSION_MINOR" != "" ]
        then
                PKG_VERSION="$PKG_VERSION.$PKG_VERSION_MINOR"
        fi

 	if [ "$PKG_VERSION_REVISION" != "" ]
        then
                PKG_VERSION="$PKG_VERSION.$PKG_VERSION_REVISION"
        fi

	if [ "$PKG_VERSION_BUILD" != "" ]
	then
		PKG_VERSION="$PKG_VERSION.$PKG_VERSION_BUILD"
	fi

	if [ ! -z  "$PKG_VERSION_RELEASE" ]
	then
		PKG_VERSION="$PKG_VERSION.$PKG_VERSION_RELEASE"
	fi

	PKG_VERSION_BASE="$PKG_VERSION_MAJOR.$PKG_VERSION_MINOR"
	if [ "$PKG_VERSION_BASE" = "." ]
	then
		PKG_VERSION_BASE=
	fi

	if [ "$PKG_VERSION_MAJOR.$PKG_VERSION_MINOR.$PKG_VERSION_REVISION.$PKG_VERSION_BUILD" = "0.0.0.0" ]
	then
		PKG_VERSION="$PKG_VERSION_NOT_FOUND"
	fi


#echo "ZZZ PKG_VERSION=$PKG_VERSION PKG_VERSION_BASE=$PKG_VERSION_BASE MAJOR=$PKG_VERSION_MAJOR MINOR=$PKG_VERSION_MINOR REVISION=$PKG_VERSION_REVISION BUILD=$PKG_VERSION_BUILD RELEASE=$PKG_VERSION_RELEASE"
}


# Concatanate PKGFILE_VERSION_MAJOR, PKGFILE_VERSION_MINOR, PKGFILE_VERSION_REVISION,
# PKGFILE_VERSION_BUILD and PKGFILE_VERSION_RELEASE together to form PKGFILE_VERSION
getFullPKGFileVersion()
{
	PKGFILE_VERSION=
	PKGFILE_VERSION_BASE=

	if [ "$PKGFILE_VERSION_MAJOR" != "" ]
        then
                PKGFILE_VERSION="$PKGFILE_VERSION_MAJOR"
        fi

	if [ "$PKGFILE_VERSION_MINOR" != "" ]
        then
                PKGFILE_VERSION="$PKGFILE_VERSION.$PKGFILE_VERSION_MINOR"
        fi

 	if [ "$PKGFILE_VERSION_REVISION" != "" ]
        then
                PKGFILE_VERSION="$PKGFILE_VERSION.$PKGFILE_VERSION_REVISION"
        fi

	if [ "$PKGFILE_VERSION_BUILD" != "" ]
	then
		PKGFILE_VERSION="$PKGFILE_VERSION.$PKGFILE_VERSION_BUILD"
	fi

	if [ ! -z  "$PKGFILE_VERSION_RELEASE"  ]
	then
		PKGFILE_VERSION="$PKGFILE_VERSION.$PKGFILE_VERSION_RELEASE"
	fi

	PKGFILE_VERSION_BASE="$PKGFILE_VERSION_MAJOR.$PKGFILE_VERSION_MINOR"
	if [ "$PKGFILE_VERSION_BASE" = "." ]
	then
		PKGFILE_VERSION_BASE=
	fi

#echo "ZZZ PKGFILE_VERSION=$PKGFILE_VERSION PKGFILE_VERSION_BASE=$PKGFILE_VERSION_BASE  MAJOR=$PKGFILE_VERSION_MAJOR MINOR=$PKGFILE_VERSION_MINOR REVISION=$PKGFILE_VERSION_REVISION BUILD=$PKGFILE_VERSION_BUILD RELEASE=$PKGFILE_VERSION_RELEASE"
}


# Get the packages dynamically and populates package names into DYN_PKG_LIST
getPackagesDynamically()
{
	# Get the packages installed and put them into DYN_PKG_LIST
	DYN_PKG_LIST=`$CMD_LIST_ALL_PACKAGES | egrep $NOVELL_PACKAGES | $CMD_PKG_NAME_EXTRACT | sort | uniq`

	# Set the main PKG_LIST to the Dynmically generated one
	PKG_LIST=$DYN_PKG_LIST

}

# Get the packages dynamically and populates package names into DYN_PKG_LIST
getProductPackagesDynamically()
{
	# Get the packages installed and put them into DYN_PKG_LIST
	DYN_PKG_LIST=`$CMD_LIST_ALL_PACKAGES | egrep $PRODUCT_PACKAGES | $CMD_PKG_NAME_EXTRACT | sort | uniq`

	# Set the main PKG_LIST to the Dynmically generated one
	PKG_LIST=$DYN_PKG_LIST
}

# Get the package files dynamically and populates file names into DYN_PKG_LIST
getPackageFilesDynamically()
{
	# Get the packages in ./$OS and put them into DYN_PKGFILE_LIST
	DYN_PKGFILE_LIST="`ls ./$OS/* 2>$NULLDEV`"

	# Set the main PKGFILE_LIST to the Dynmically generated one
	PKGFILE_LIST=$DYN_PKGFILE_LIST

	# If no packages were found then display error and summary then exit.
	if [ -z "$PKGFILE_LIST" ]
	then
		dispError "No package files were found in ./$OS/* !"
		failure
		stageAbort
	fi
}

# Remove package from PKG_LIST
# This is for different installation scenarios
removePackageFromPKGList()
{
	TEMP_PKG_LIST=

	for TEMP_PKG in $PKG_LIST
	do
		if [ ! "`echo $TEMP_PKG | grep $1`" ]
		then
			TEMP_PKG_LIST="$TEMP_PKG_LIST $TEMP_PKG"
		fi
	done
	
	PKG_LIST=$TEMP_PKG_LIST
#echo TTT New PKG List = $PKG_LIST

}

# Remove package from PKGFILE_LIST
# This is for different installation scenarios
removePackageFromPKGFILEList()
{
	TEMP_PKGFILE_LIST=

	for TEMP_PKGFILE in $PKGFILE_LIST
	do
		if [ ! "`echo $TEMP_PKGFILE | grep $1`" ]
		then
			TEMP_PKGFILE_LIST="$TEMP_PKGFILE_LIST $TEMP_PKGFILE"
		fi
	done
	
	PKGFILE_LIST=$TEMP_PKGFILE_LIST
#echo TTT New PKGFILE List = $PKGFILE_LIST

}

# Modify the package list if needed. Such as for different installation
# scenarios
modifyPKGList()
{
	# Modify the PKG_LIST for iManager 
	if [ "$INSTALLATION_SCENARIO" = "$PRODUCT_REQUIRED_IMANAGER_TITLE" ]
	then
		for REM_PKG in $PRODUCT_REQUIRED_IMANAGER_PKG_EXCLUDE_LIST
		do
			removePackageFromPKGList $REM_PKG
		done
	fi
}

# Modify the package file list if needed. Such as for different installation
# scenarios
modifyPKGFILEList()
{
	# Modify the PKGFILE_LIST for iManager 
	if [ "$INSTALLATION_SCENARIO" = "$PRODUCT_REQUIRED_IMANAGER_TITLE" ]
	then
		for REM_PKG in $PRODUCT_REQUIRED_IMANAGER_PKG_EXCLUDE_LIST
		do
			removePackageFromPKGFILEList $REM_PKG
		done
	fi
}


# Get the package files dynamically and then get the package name and populates package names into DYN_PKG_LIST
getPackagesDynamicallyBasedOnPackageFiles()
{
	PKG_LIST=

	# Get the packages in ./$OS and put them into DYN_PKGFILE_LIST
	DYN_PKGFILE_LIST="`ls ./$OS/* 2>$NULLDEV`"

	PKGFILE_LIST=
	# Loop through the package files found in $DYN_PKGFILE_LIST and put the package names into PKGFILE_LIST
	for PKGFILE in $DYN_PKGFILE_LIST
	do
		getPackageNameFromPackageFile "$PKGFILE"			
		if [ ! -z "$PKGFILE_PKG_NAME" ]
		then
			PKG_LIST="$PKG_LIST $PKGFILE_PKG_NAME"
		fi
	done

	# If no packages were found then display error and summary then exit.
	if [ -z "$PKG_LIST" ]
	then
		dispError "No package files were found in ./$OS/* !"
		failure
		stageAbort
	fi
}

# Verify that the packages in the ./$OS directory are actually packages that
# are for this patch (based on the $PRODUCT_VERSION_BASE).
verifyPackageFilesBaseVersion()
{
	dispHeader "Package File Name" "Package Version" "YES"
	getPackageFilesDynamically

	# Loop through each package found and get the version info and check
	# that it matches $PRODUCT_VERSION_BASE
	for PKGFILE in $PKGFILE_LIST
	do
		getPackageFileVersionInfo "$PKGFILE"
		dispItem "$PKGFILE" "$PKGFILE_VERSION" "YES"
	
		# Check to ensure that the base version is the same
		# as what is required to be installed.

		if [ "$PKGFILE_VERSION_MAJOR" = "$PRODUCT_VERSION_MAJOR" ]
		then
			if [ "$PKGFILE_VERSION_MINOR" -ge "$PRODUCT_VERSION_MINOR" ]
			then
				dispResult "     OK      "
			fi
		else
			dispResult "${fgred}WRONG VERSION${reset}"
			failure
		fi
	done
}

# Verify that the installed package versions are the right ones to install
# this patch over  (based on the $PRODUCT_VERSION_BASE).
verifyInstalledPackagesBaseVersion()
{
	dispHeader "Installed Package Name" "Package Version"

	# Get the files from ./$OS because these are the packages we are checking
	getPackageFilesDynamically

	# Loop through each package file found and get the installed package 
	# version info and check that it matches $PRODUCT_VERSION_BASE
	for PKGFILE in $PKGFILE_LIST
	do
		getPackageFileVersionInfo "$PKGFILE"
		getPackageNameFromPackageFile "$PKGFILE"
		getPackageVersionInfo "$PKGFILE_PKG_NAME"



		# If $PKG_VERSION was not filled in (because of an error or the package wasn't installed) then still
		# display the | bar to seperate the columns - this is for cosmetic reasons (ie: to make it look good still).
		if [ -z "$PKG_VERSION" ]
		then
			dispItem "$PKGFILE_PKG_NAME" " "
		else
			dispItem "$PKGFILE_PKG_NAME" "$PKG_VERSION"
		fi

		# Check to ensure that the base version is the same as what is required to be installed.
		if [ "$PKG_VERSION_BASE" -ge "$PRODUCT_VERSION_BASE" ]
		then
			# If the installed package revision is less than the one I'm installing then that is good.
			if [ "$PKG_VERSION_REVISION" -lt "$PRODUCT_VERSION_REVISION" ]
			then
				dispResult "     OK      "
			fi

			# If the installed package revision is more than the one I'm installing then that is not good as it means that
			# a newer revision has been installed, so I shall fail on this.
			if [ "$PKG_VERSION_REVISION" -gt "$PRODUCT_VERSION_REVISION" ]
                        then
                                dispResult "${fgred}NEWER VERSION${reset}"
				dispError "Existing package \"$PKGFILE_PKG_NAME $PKG_VERSION\" is a newer version than \"$PKGFILE_PKG_NAME $PKGFILE_VERSION\"\nwhich is to be installed. To downgrade the this particular package you will need\nto force the installation to continue past any errors by using the --force switch."
				failure
                        fi

			# If the installed package revision is the same as the one I'm installing then that means the package I'm	
			# installing is the same. I'll error on this and require a FORCE to allow the patch to install over the top.
			if [ "$PKG_VERSION_REVISION" = "$PRODUCT_VERSION_REVISION" ]
                        then
                                dispResult "${fgred}EQUAL VERSION${reset}"
				dispError "\"$PKGFILE_PKG_NAME $PKGFILE_VERSION\" is already installed on this system.\nTo reapply this particular package you will need to force the installation to\ncontinue past any errors by using the --force switch."
                                failure
                        fi
		else
			# Is there no $PRODUCT_VERSION_BASE at all?  If so then package most likely isn't installed, which isn't good.
			if [ -z "$PKG_VERSION_BASE" ]
			then
				dispResult "${fgred}NOT INSTALLED${reset}"				
				dispError "\"$PKGFILE_PKG_NAME\" is not installed on this system. You will need to force the\ninstallation to continue past any errors by using the --force switch."
				failure
			else
				dispResult "${fgred}WRONG PRODUCT${reset}"
				dispError "The currently installed version of \"$PKGFILE_PKG_NAME $PKG_VERSION\" is not able to have the \"$PKGFILE_PKG_NAME $PKGFILE_VERSION\" package from $PRODUCT_TITLE $PRODUCT_VERSION applied over it.\nPlease ensure you are currently running $PRODUCT_TITLE $PRODUCT_VERSION_BASE."
				failure
			fi
		fi
	done
}

# Forcefully Remove the packages
removePackages()
{
	dispHeader "Package Name To Remove"

#	NMAS will base the files it is removing from the $PRODUCT_PACKAGES regexp
#	getProductPackagesDynamically
#	getPackagesDynamicallyBasedOnPackageFiles	

	# Loop through each package file found and get the package name then remove the package
	for PKG in $PKG_LIST
	do
#		getPackageNameFromPackageFile "$PKGFILE"
		dispItem "$PKG"

		# Remove the package forcefully.  Save the stderr info to a varaible in case the package remove failed. 
		# Then the error output can be given to the installer to help them troubleshoot why the package couldn't be removed.
                REMOVED=
                $CMD_SINGLE_PKG_REMOVE $PKG 1>$COMMAND_OUTPUT_TEMP_FILE 2>$COMMAND_OUTPUT_TEMP_FILE
                REMOVED=$?

	        if [ "$REMOVED" = "0" ]
 	        then
			dispResult "   REMOVED   "				
                else
                	dispResult "${fgred}REMOVE FAILED${reset}"
			getCommandOutput
			dispError "The package \"$PKG\" could not be removed.\nPlease remove this package manually.\nThe error output is:\n${reset}$ERROR_OUTPUT"
	                failure
                fi
	done
}

# Checks for pre-requisites (eg: Min package versions)
# Input: $1=Required Package Name
#	 $2=Required Package Title
#	 $3=Required Package Major Versione
#	 $4=Required Package Minor Version
#	 $5=Required Package Revision Version
#	 $6=Required Package Build Version
checkPrerequisites()
{
	dispHeader "Package" "Version"

	ensureMinimumePackageVersionInstalled "$PRODUCT_REQUIRED_NICI_PKG_NAME" "$PRODUCT_REQUIRED_NICI_TITLE" $PRODUCT_REQUIRED_NICI_PKG_VERSION_MAJOR $PRODUCT_REQUIRED_NICI_PKG_VERSION_MINOR $PRODUCT_REQUIRED_NICI_PKG_VERSION_REVISION $PRODUCT_REQUIRED_NICI_PKG_VERSION_BUILD "REQUIRED"
#echo RRR $RC
	ensureMinimumePackageVersionInstalled "$PRODUCT_REQUIRED_IMANAGER_PKG_NAME" "$PRODUCT_REQUIRED_IMANAGER_TITLE" $PRODUCT_REQUIRED_IMANAGER_PKG_VERSION_MAJOR $PRODUCT_REQUIRED_IMANAGER_PKG_VERSION_MINOR $PRODUCT_REQUIRED_IMANAGER_PKG_VERSION_REVISION $PRODUCT_REQUIRED_IMANAGER_PKG_VERSION_BUILD "OPTIONAL"
#echo RRR $RC
	if [ "$RC" = "0" ]
	then	
		INSTALLATION_SCENARIO="$PRODUCT_REQUIRED_IMANAGER_TITLE"
	fi
	
	ensureMinimumePackageVersionInstalled "$PRODUCT_REQUIRED_NDS_PKG_NAME" "$PRODUCT_REQUIRED_NDS_TITLE" $PRODUCT_REQUIRED_NDS_PKG_VERSION_MAJOR $PRODUCT_REQUIRED_NDS_PKG_VERSION_MINOR $PRODUCT_REQUIRED_NDS_PKG_VERSION_REVISION $PRODUCT_REQUIRED_NDS_PKG_VERSION_BUILD "OPTIONAL"
#echo RRR $RC
	if [ "$RC" = "0" ]
	then	
		INSTALLATION_SCENARIO="$PRODUCT_REQUIRED_NDS_TITLE"
	fi

	# Check for eDirectory 8.7.1 and if it is on Linux 
	# use a different init script
	if [ "$PKG_VERSION_REVISION" = 1 ]
	then
		if [ "$OS" = "Linux" ]	
		then	
			PRODUCT_INIT_SCRIPT="$PRODUCT_INIT_SCRIPT_871"
		fi
	fi

	dispItem "Installation Scenario" "$INSTALLATION_SCENARIO "

	if [ -z "$INSTALLATION_SCENARIO" ]
	then
               	dispResult "${fgred}   FAILED    ${reset}"
		dispError "This patch must be installed on at least $PRODUCT_REQUIRED_NICI_TITLE $PRODUCT_REQUIRED_NICI_PKG_VERSION with either $PRODUCT_REQUIRED_IMANAGER_TITLE $PRODUCT_REQUIRED_IMANAGER_PKG_VERSION and/or $PRODUCT_REQUIRED_NDS_TITLE $PRODUCT_REQUIRED_NDS_PKG_VERSION."
		failure
	else
		dispResult "     OK     "
	fi
}

# Ensure that a minimum version of package specified has been installed
# Input: $1=Required Package Name
#	 $2=Required Package Title
#	 $3=Required Package Major Versione
#	 $4=Required Package Minor Version
#	 $5=Required Package Revision Version
#	 $6=Required Package Build Version
#        $7=Required or Optional?   "REQUIRED" "OPTIONAL"
ensureMinimumePackageVersionInstalled()
{
	PACKAGE_NAME_TO_CHECK="$1"
	PRODUCT_REQUIRED_TITLE="$2"
	PRODUCT_REQUIRED_PKG_VERSION_MAJOR=$3
	PRODUCT_REQUIRED_PKG_VERSION_MINOR=$4
	PRODUCT_REQUIRED_PKG_VERSION_REVISION=$5
	PRODUCT_REQUIRED_PKG_VERSION_BUILD=$6
	PRODUCT_REQUIRED_VERSION="$PRODUCT_REQUIRED_PKG_VERSION_MAJOR.$PRODUCT_REQUIRED_PKG_VERSION_MINOR.$PRODUCT_REQUIRED_PKG_VERSION_REVISION"
	PRODUCT_REQ_OR_OPT="$7"
	MIN_PKG_VERSION_FAILED="false"
	RC=99
	# Return codes:
	#0 = Package exists
	#1 = Optional package does not exist
	#2 = Failed, optional package is installed but min version not met
	#3 = Failed, required package is installed but min version not met
	#99 = Return code was not accounted for. This must be a logic error in the code.

#echo EEE \"$PACKAGE_NAME_TO_CHECK\" needs to be at least PRODUCT_REQUIRED_VERSION=$PRODUCT_REQUIRED_VERSION $PRODUCT_REQ_OR_OPT
	getPackageVersionInfo "$PACKAGE_NAME_TO_CHECK"
	dispItem "$PRODUCT_REQUIRED_TITLE ($PACKAGE_NAME_TO_CHECK)" "$PKG_VERSION"	

	if [ "$PKG_VERSION" != "$PKG_VERSION_NOT_FOUND" ]
	then
	 	if [ "$PKG_VERSION_MAJOR" -gt "$PRODUCT_REQUIRED_PKG_VERSION_MAJOR" ]	
		then

		    MIN_PKG_VERSION_FAILED="false"

		elif [ "$PKG_VERSION_MAJOR" -eq "$PRODUCT_REQUIRED_PKG_VERSION_MAJOR" ]	
		then
	
			if [ "$PKG_VERSION_MINOR" -gt "$PRODUCT_REQUIRED_PKG_VERSION_MINOR" ]		
			then

			    MIN_PKG_VERSION_FAILED="false"

			elif [ "$PKG_VERSION_MINOR" -eq "$PRODUCT_REQUIRED_PKG_VERSION_MINOR" ]		
			then
				if [ "$PKG_VERSION_REVISION" -gt "$PRODUCT_REQUIRED_PKG_VERSION_REVISION" ]		
				then

				    MIN_PKG_VERSION_FAILED="false"
				    
				elif [ "$PKG_VERSION_REVISION" -eq "$PRODUCT_REQUIRED_PKG_VERSION_REVISION" ]		
				then
					if [ "$PKG_VERSION_BUILD" -ge "$PRODUCT_REQUIRED_PKG_VERSION_BUILD" ]		
					then
						MIN_PKG_VERSION_FAILED="false"
					else
						MIN_PKG_VERSION_FAILED="true"					
					fi					
				else
					MIN_PKG_VERSION_FAILED="true"					
				fi		
			else
				MIN_PKG_VERSION_FAILED="true"
			fi
		else
			MIN_PKG_VERSION_FAILED="true"
		fi
		
		# The package must be installed because we have obtained
		# some version info. But was it the min version?
		if [ "$MIN_PKG_VERSION_FAILED" = "false" ]
		then	
			dispResult "     OK      "
			RC=0
		else
			dispResult "${reset}${bgred}${fgblack}   FAILED    ${reset}"
			dispError "The minimum version of \"$PRODUCT_REQUIRED_TITLE\" that $PRODUCT_TITLE $PRODUCT_VERSION can be\ninstalled onto is \"$PRODUCT_REQUIRED_VERSION\". Please install \"$PRODUCT_REQUIRED_TITLE $PRODUCT_REQUIRED_VERSION\" or higher\nbefore applying this patch."
			failure
			RC=2
		fi
	else
		# If the product is required and we didn't get a version
		# number then fail, otherwise it must be an optional package
		if [ "$PRODUCT_REQ_OR_OPT" = "REQUIRED" ]
		then
			dispResult "${reset}${bgred}${fgblack}   FAILED    ${reset}"
			dispError "The minimum version of \"$PRODUCT_REQUIRED_TITLE\" that $PRODUCT_TITLE $PRODUCT_VERSION can be\ninstalled onto is \"$PRODUCT_REQUIRED_VERSION\". Please install \"$PRODUCT_REQUIRED_TITLE $PRODUCT_REQUIRED_VERSION\" or higher\nbefore applying this patch."
			failure
			RC=3
		else
			dispResult "NOT INSTALLED"		
			RC=1
		fi
	fi

	return $RC
}

# Install the packages
installPackages()
{
	dispHeader "Package Name To Install"

	# Get the files from ./$OS because these are the packages we are going to install
#	getPackageFilesDynamically
	
	# Loop through each package file found and get the package name then install it
	for PKGFILE in $PKGFILE_LIST
	do
		getPackageNameFromPackageFile "$PKGFILE" "AIX_INSTALL_ALL"
	
		if [ "$CMD_SINGLE_PKG_INSTALL_INGORE_PKG_NAME" = "true" ]
		then	
			PKGFILE_PKG_NAME=
		fi	

		dispItem "$PKGFILE"

		# Install the package and capture the error output into $COMMAND_OUTPUT_TEMP_FILE                
		INSTALLED=

#echo ZZZ        $CMD_SINGLE_PKG_INSTALL `pwd`/$PKGFILE $PKGFILE_PKG_NAME
		# The "echo y" is so that on solaris the prompt for the license
		# agreement with the ntls package is accepted and installed
		# automatically.
                echo y | $CMD_SINGLE_PKG_INSTALL `pwd`/$PKGFILE $PKGFILE_PKG_NAME 1>$COMMAND_OUTPUT_TEMP_FILE 2>$COMMAND_OUTPUT_TEMP_FILE

                INSTALLED=$?

                if [ "$INSTALLED" = "0" ]
 	        then
			dispResult "  INSTALLED  "				
                else
                	dispResult "${fgred}INSTALL ERROR${reset}"
			getCommandOutput
			dispError "The package \"$PKGFILE\" could not be installed. Please install this package manually.\nThe error output is:\n${reset}$ERROR_OUTPUT"
	                failure
                fi
	done
}



# This stops (kills) the processes so that they aren't running when the packages are removed.
# Though the installer should already have done this gracefully before hand as the warning message
# told them to do.
stopProcesses()
{
        dispHeader "Process name"

        for PROC_NAME in $PROCESS_LIST
        do
                dispItem $PROC_NAME
                PIDS="`ps -ef | grep \"$PROC_NAME\" | grep -v grep | awk '{ print $2 }'`"

                if [ "$PIDS" = "" ]
                then
                        KILLSTATUS=" NOT RUNNING "
                else
                        kill -KILL $PIDS 1>$NULLDEV 2>$NULLDEV
                        KILLED=$?
                        if [ "$KILLED" = "0" ]
                        then
                                KILLSTATUS="   STOPPED   "
                        else
                		KILLSTATUS="${fgred} NOT STOPPED ${reset}"
		               	failure
				dispError "Process \"$PROC_NAME (PIDS=$PIDS)\" could not be stopped. Please ensure that Novell eDirectory has been stopped before running this installation."
                        fi
                fi

                dispResult "$KILLSTATUS"
        done
}

# Ensure that the user is root
ensureUserIsRoot()
{
	dispHeader "User ID"
	USER_ID="`$CMD_USER_ID | awk '{print $1}' | awk -F\"=\" '{print $2}' | awk -F\"(\" '{print $1}'`"

	dispItem "uid=$USER_ID"

	if [ "$USER_ID" != "0" ] 
	then
		dispResult "${fgred}   NOT ROOT  ${reset}"
		dispError "You must be 'root' (uid=0) to install $PRODUCT_TITLE $PRODUCT_VERSION"
		failure
	else
		dispResult "     OK      "
	fi
}


# Backup / Restore files
# Input $1="BACKUP" or "RESTORE"
configFiles()
{
	case $1 in
       	        "BACKUP")	ACTION_TO_PERFORM=backupFile ;;
       	        "RESTORE")	ACTION_TO_PERFORM=restoreFile ;;
	        *)		dispError "Internal Error:  configFiles() was passed an invalid or missing parameter.";
				abortScript "1" "YES" ;;
	esac

	dispHeader "File Name"

	# Backup or restore these files. 1st param is source dir, 2nd param is source file
	$ACTION_TO_PERFORM "$PRODUCT_CONF_DIR" "$PRODUCT_NDSCONF_FILE"
	$ACTION_TO_PERFORM "$PRODUCT_CONF_DIR" "$PRODUCT_IMONCONF_FILE"
	$ACTION_TO_PERFORM "$PRODUCT_DIB_DIR" "$PRODUCT_NDSDBINI_FILE"
}

# This will backup (rename) the file and append the product version to it
# Input: $1=source dir
#        $2=source file
backupFile()
{
	FILE_TO_BACKUP="$1/$2"
	dispItem "$FILE_TO_BACKUP"
		
	if [ -f "$FILE_TO_BACKUP" ]
	then
		mv "$FILE_TO_BACKUP" "$FILE_TO_BACKUP.$PRODUCT_VERSION.backup"
		RESULT=$?
		if [ "$RESULT" = "0" ]
		then
			dispResult "     OK      "
		else
			dispResult "${fgred}    ERROR    ${reset}"			
		fi
	else
		dispResult "FILE != EXIST"			
	fi
}

# This will rename back the file removing the PRODUCT_VERSION off it as long as another one wasn't in it's place
# Input: $1=source dir
#        $2=source file
restoreFile()
{
	FILE_TO_RESTORE="$1/$2"
	dispItem "$FILE_TO_RESTORE.$PRODUCT_VERSION.backup"
		
	if [ -f "$FILE_TO_RESTORE" ]
	then
		if [ -f "$FILE_TO_RESTORE.$PRODUCT_VERSION.backup" ]
		then
			mv "$FILE_TO_RESTORE" "$FILE_TO_RESTORE.$PRODUCT_VERSION.default"
		fi
	fi
	
	if [ -f "$FILE_TO_RESTORE.$PRODUCT_VERSION.backup" ]
	then	
		mv "$FILE_TO_RESTORE.$PRODUCT_VERSION.backup" "$FILE_TO_RESTORE"
		RESULT=$?	
		if [ "$RESULT" = "0" ]
		then
			dispResult "     OK      "
		else
			dispResult "${fgred}    ERROR    ${reset}"			
		fi
	else
		dispResult "FILE != EXIST"
	fi
}

##### End of Action Functions #####

# Check the command line to see if any command line switches were provided
checkCommandline()
{
	while [ "$#" -gt "0" ]
	do
		case $1 in
        	        "-h" | "--help")          dispCommandlineHelp ;;
			"-f" | "--force")	  FORCE="true" ;;
			"-n" | "--noansi")        setANSIColours "OFF";;
			"-s" | "--showall")	  SHOWALL_PACKAGES="true" ;;
			"-i" | "--showinstalled") SHOWINSTALLED_PACKAGES="true" ;;
			"-ne"| "--noerrors")	  NO_ERRORS="true" ;;

		        *)			  dispCommandlineHelp ;;
		esac
		shift
	done     

	# Show the packages that are installed, along with their respective package version numbers.
	if [ "$SHOWALL_PACKAGES" = "true" ]
	then
		dispTitle
		stageStart "Installed Novell Packages"
		getPackagesDynamically
		showPackagesInstallStatus
		stageEnd
		exit 0
	fi

	# Show the packages that are installed, along with their respective package version numbers.
	# This is for the packages from the ./$OS directory.  It's a quick way to see if the server
	# is already up to date with the package versions.
	if [ "$SHOWINSTALLED_PACKAGES" = "true" ]
	then
		dispTitle
		stageStart "Installed Novell Packages"
		getPackagesDynamicallyBasedOnPackageFiles
		showPackagesInstallStatus
		stageEnd
		exit 0
		
	fi
}

# Display the command line help
dispCommandlineHelp()
{
	setANSIColours "OFF"
	dispTitle
	echo "Syntax: $0 [options]"
	echo "Where [options] is one of the following switches:"
	echo
	echo "  -f   --force"
	echo "       Force the installation of the patch at all costs (continues on failure)."
	echo 
	echo "  -s   --showall"
	echo "       Display the package versions that are installed for Novell products."
	echo
	echo "  -i   --showinstalled"
	echo "       Display the installed package versions for the packages that are"
	echo "       included in this patch."
	echo
	echo "  -n   --noansi"
	echo "       Disable the use of ANSI colours."
	echo
	echo "  -ne  --noerrors"
	echo "       Disable the display of expanded error messages that are displayed"
	echo "       when an error is generated."
	echo
	echo "  -h   --help"
	echo "       Displays this command line help."
	echo
	exit 2
}


# Main function
main()
{
	setVars_General
	registerSignalTraps
	getHostOS
	verifySupportedOS
	setVars_Product
	getVars_Product
	setVars_$OS
	setANSIColours "ON"

	checkCommandline $1 $2 $3 $4 $5 $6 $7 $8 $9

	dispTitle
	dispForceMessageIfRequired
	dispInstallationPreparationMessage
	promptForYesOrNo "Are you sure you wish to continue?"
	dispProceedMessage

	# Perform the main operations (in 'stages'). Each stage number is incremented dynamically as required.

	# Ensure that the user is 'root' eg: uid=0.
	stageStart "Checking To Ensure You Are 'root'..."
	ensureUserIsRoot
	stageEnd

	# The security update on *nix requires that the eDirectory 8.7.1 and NICI 2.6.7 is installed
	stageStart "Checking for pre-requisites..."
	checkPrerequisites
	stageEnd

	# Kill any of the Novell daemons/services that may be running before continuing with the
	# installation.
	stageStart "Shutting Down Novell Daemons..."
	stopProcesses
	stageEnd

#	# Backup certain config files
#	stageStart "Backing Up Config Files..."
#	configFiles "BACKUP"
#	stageEnd

	# Force the removal of the installed packages if needed
	# Some platforms may just be able to be upgraded without
	# removing the packages previously
	if [ "$REMOVE_PACKAGES" = "true" ]
	then
		stageStart "Removing Installed Packages..."
		getPackagesDynamicallyBasedOnPackageFiles	
		modifyPKGList
		removePackages
		stageEnd
	fi

	# Install the packages from the ./$OS directory
	stageStart "Installing New Packages From \"./$OS\"..."
	getPackageFilesDynamically
	modifyPKGFILEList
	installPackages
	stageEnd

	# Show the versions of the installed packages corresponding to the packages in ./$OS
	stageStart "Newly Installed Package Versions..."
	getPackagesDynamicallyBasedOnPackageFiles
	modifyPKGList
	showPackagesInstallStatus
	stageEnd

#	# Restore certain config files
#	stageStart "Restoring Config Files..."
#	configFiles "RESTORE"
#	stageEnd

	dispSummaryInfo
}

main $1 $2 $3 $4 $5 $6 $7 $8 $9
